#include "app_cfg.h"
#include "HW.h"
#include "fifo.h"
#include "PowerExt.h"

#include "usb_lib.h"
#if USB_PROTOCOL == USB_ST_VCOM
#	include "hw_config.h"
#	include "usb_conf.h"
#	include "usb_desc.h"
#	include "usb_cdc.h"
#elif (USB_PROTOCOL == USB_AT_JTAGICE_MKII) || (USB_PROTOCOL == USB_AT_DRAGON)
#	include "hw_config_at.h"
#	include "usb_conf_at.h"
#	include "usb_desc_at.h"
#	include "usb_cdc_at.h"
#endif
#define USB_DATA_SIZE		BULK_MAX_PACKET_SIZE

#if (	((USB_PROTOCOL == USB_AT_JTAGICE_MKII) || (USB_PROTOCOL == USB_AT_DRAGON))	\
		&& (USB_WITH_CDC == USB_WITH_IAD_CDC) && USB_WITH_MASSSTORAGE)				\
	|| ((USB_PROTOCOL == USB_ST_VCOM) && USB_WITH_MASSSTORAGE)
#	include "mass_mal.h"
#endif

void GPIO_Dir(GPIO_TypeDef* GPIOx, uint8 mode, uint8 pin)
{
	uint32 tmp_reg;

	if(pin < 8)
	{
		tmp_reg = GPIOx->CRL;
		tmp_reg &= ~(((u32)0x0F) << ((pin - 0) * 4));
		tmp_reg |= (u32)(mode & 0x0F) << ((pin - 0) * 4);
		GPIOx->CRL = tmp_reg;
	}
	else
	{
		tmp_reg = GPIOx->CRH;
		tmp_reg &= ~(((u32)0x0F) << ((pin - 8) * 4));
		tmp_reg |= (u32)(mode & 0x0F) << ((pin - 8) * 4);
		GPIOx->CRH = tmp_reg;
	}

	if(mode & 0x20)
	{
		if(mode & 0x10)
		{
			GPIOx->BSRR = (((u32)0x01) << pin);
		}
		else
		{
			GPIOx->BRR = (((u32)0x01) << pin);
		}
	}
}

/*******************************************************************************
* Function Name	: Sys_Init
* Description		: Initialize the system
* Input					: None
* Output				 : None
* Return				 : None
*******************************************************************************/
void Sys_Init(void)
{
	/* Configure the system clocks */
	RCC_Configuration();

	/* NVIC Configuration */
	NVIC_Configuration();

	/* Configure the GPIO ports */
	GPIO_Configuration();

	/* Configure the ADC ports */
	ADC_Configuration();

	/* Configure external power */
	PWREXT_INIT();

#if (	((USB_PROTOCOL == USB_AT_JTAGICE_MKII) || (USB_PROTOCOL == USB_AT_DRAGON))	\
		&& (USB_WITH_CDC == USB_WITH_IAD_CDC) && USB_WITH_MASSSTORAGE)				\
	|| ((USB_PROTOCOL == USB_ST_VCOM) && USB_WITH_MASSSTORAGE)
	MAL_Init(0);
#endif
	USB_Init();
}

/*******************************************************************************
* Function Name	: RCC_Configuration
* Description		: Configures the different system clocks.
* Input					: None
* Output				 : None
* Return				 : None
*******************************************************************************/
void RCC_Configuration(void)
{
	/* RCC system reset(for debug purpose) */
	RCC_DeInit();

	/* Enable HSE */
	RCC_HSEConfig(RCC_HSE_ON);

	/* Wait till HSE is ready */
	if(RCC_WaitForHSEStartUp() == SUCCESS)
	{
		/* Enable Prefetch Buffer */
		FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

		/* Flash 2 wait state */
		FLASH_SetLatency(FLASH_Latency_2);
 	
		/* HCLK = SYSCLK */
		RCC_HCLKConfig(RCC_SYSCLK_Div1); 
	
		/* PCLK2 = HCLK */
		RCC_PCLK2Config(RCC_HCLK_Div1); 

		/* PCLK1 = HCLK/2 */
		RCC_PCLK1Config(RCC_HCLK_Div2);

		/* PLLCLK */
		RCC_PLLConfig(RCC_PLLSource_HSE_Div1, (_SYS_FREQUENCY * 1000000 / HSE_Value - 2) << 18);

		/* Enable PLL */ 
		RCC_PLLCmd(ENABLE);

		/* Wait till PLL is ready */
		while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET)
		{
		}

		/* Select PLL as system clock source */
		RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

		/* Wait till PLL is used as system clock source */
		while((RCC_GetSYSCLKSource() >> 2) != RCC_SYSCLKSource_PLLCLK)
		{
		}
	}

	/* USBCLK = PLLCLK / 1.5 */
#if _SYS_FREQUENCY == 72
	RCC_USBCLKConfig(RCC_USBCLKSource_PLLCLK_1Div5);
#elif _SYS_FREQUENCY == 48
	RCC_USBCLKConfig(RCC_USBCLKSource_PLLCLK_Div1);
#else
#	error _SYS_FREQUENCY not supported
#endif

	// DMA Clock
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOC |
						   RCC_APB2Periph_SPI1 | RCC_APB2Periph_USART1 | RCC_APB2Periph_AFIO | RCC_APB2Periph_ADC1, ENABLE);

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2 | RCC_APB1Periph_USB, ENABLE);
}

/*******************************************************************************
* Function Name	: GPIO_Configuration
* Description		: Configures the different GPIO ports.
* Input					: None
* Output				 : None
* Return				 : None
*******************************************************************************/
void GPIO_Configuration(void)
{
	// Disable JTAG
	GPIO_PinRemapConfig(GPIO_Remap_SWJ_Disable, ENABLE);

	// Enable pin
	GLOBAL_OUTPUT_DISABLE();
	GLOBAL_OUTPUT_INIT();
	SW_SETINPUT();
	SW_DIR_INIT();

	// Key Init
	BKP_TamperPinCmd(DISABLE);
	KEY_Init();

	// LED Init
	LED_ALL_OFF();
	LED_Init();

	// VSample Init
	GPIO_Dir(TVCC_PORT, GPIO_MODE_AIN, TVCC_PIN);

	// Remap for SPI1
	GPIO_PinRemapConfig(GPIO_Remap_SPI1, ENABLE);
}

/*******************************************************************************
* Function Name	: NVIC_Configuration
* Description		: Configures Vector Table base location.
* Input					: None
* Output				 : None
* Return				 : None
*******************************************************************************/
void NVIC_Configuration(void)
{
	NVIC_InitTypeDef NVIC_InitStructure;

	NVIC_DeInit();

#ifdef	VECT_TAB_RAM	
	/* Set the Vector Table base location at 0x20000000 */ 
	NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else	/* VECT_TAB_FLASH	*/
	/* Set the Vector Table base location at 0x08000000 */ 
//	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);	 
	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x2000);
#endif

	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

	/* Enable the USB Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = USB_LP_CAN_RX0_IRQChannel;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
}

void Enable_CDC_Int(void)
{
	// enable USB Int.
	NVIC->ISER[(USB_LP_CAN_RX0_IRQChannel >> 0x05)] =
		(u32)0x01 << (USB_LP_CAN_RX0_IRQChannel & (u8)0x1F);

#if USB_CDC_TARGET_IF == USB_CDC_ON_USART
	// CDC on USART
	// enable USART Int.
	NVIC->ISER[(USART1_IRQChannel >> 0x05)] =
		(u32)0x01 << (USART1_IRQChannel & (u8)0x1F);
#endif
}

void Disable_CDC_Int(void)
{
	// disable USB Int.
	NVIC->ICER[(USB_LP_CAN_RX0_IRQChannel >> 0x05)] =
		(u32)0x01 << (USB_LP_CAN_RX0_IRQChannel & (u8)0x1F);

#if USB_CDC_TARGET_IF == USB_CDC_ON_USART
	// CDC on USART
	// disable USART Int.
	NVIC->ICER[(USART1_IRQChannel >> 0x05)] =
		(u32)0x01 << (USART1_IRQChannel & (u8)0x1F);
#endif
}

/*******************************************************************************
* Function Name	: SPI_Configuration
* Description		: Configures SPI Interface.
* Input					: None
* Output				 : None
* Return				 : None
*******************************************************************************/
void SPI_Configuration(SPI_TypeDef* SPIx,u16 mode,u16 brp,u16 fb,u16 cpol,u16 cpha)
{
//	DeInit cannot be called here because this function will be called to set the speed of SPI
//	SPI_I2S_DeInit(SPIx);

	SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
	SPI_InitStructure.SPI_Mode = mode;
	SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
	SPI_InitStructure.SPI_CPOL = cpol;
	SPI_InitStructure.SPI_CPHA = cpha;
	SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
	SPI_InitStructure.SPI_BaudRatePrescaler = brp;
	SPI_InitStructure.SPI_FirstBit = fb;
	SPI_InitStructure.SPI_CRCPolynomial = 1;
	SPI_Init(SPIx, &SPI_InitStructure);

	SPI_Cmd(SPIx, ENABLE);
}

void JTAG_DMA_Fini(void)
{
	DMA_DeInit(JTAG_TAP_HS_SPI_M_RX_DMA);
	DMA_DeInit(JTAG_TAP_HS_SPI_M_TX_DMA);
	DMA_DeInit(JTAG_TAP_HS_SPI_S_TX_DMA);
}

void JTAG_DMA_Init(void)
{
	DMA_InitTypeDef  DMA_InitStructure;

	DMA_InitStructure.DMA_PeripheralBaseAddr = (u32)&JTAG_TAP_HS_SPI_M->DR;
	DMA_InitStructure.DMA_MemoryBaseAddr = (uint32)0;
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
	DMA_InitStructure.DMA_BufferSize = 0;
	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
	DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
	DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
	DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
	DMA_Init(JTAG_TAP_HS_SPI_M_RX_DMA, &DMA_InitStructure);

	DMA_InitStructure.DMA_PeripheralBaseAddr = (u32)&JTAG_TAP_HS_SPI_S->DR;
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
	DMA_Init(JTAG_TAP_HS_SPI_S_TX_DMA, &DMA_InitStructure);
}

/*******************************************************************************
* Function Name	: ADC_Configuration
* Description		: Configures SPI Interface.
* Input					: None
* Output				 : None
* Return				 : None
*******************************************************************************/
void ADC_Configuration(void)
{
	ADC_InitTypeDef ADC_InitStructure;

	ADC_DeInit(ADC1);

	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
	ADC_InitStructure.ADC_ScanConvMode = ENABLE;
	ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = 1;
	ADC_Init(ADC1, &ADC_InitStructure);

	/* ADC1 regular channel7 configuration */ 
	ADC_RegularChannelConfig(ADC1, TVCC_ADC_CHANNEL, 1, ADC_SampleTime_55Cycles5);

	/* Enable ADC1 */
	ADC_Cmd(ADC1, ENABLE);

	/* Enable ADC1 reset calibaration register */   
	ADC_ResetCalibration(ADC1);
	/* Check the end of ADC1 reset calibration register */
	while(ADC_GetResetCalibrationStatus(ADC1));

	/* Start ADC1 calibaration */
	ADC_StartCalibration(ADC1);
	/* Check the end of ADC1 calibration */
	while(ADC_GetCalibrationStatus(ADC1));

	/* Start ADC1 Software Conversion */ 
	ADC_SoftwareStartConvCmd(ADC1, ENABLE);
}

void USB_Init_SerialString(uint8 *strSerial, uint16 len)
{
	uint8 *chip_serial = (uint8*)0x1FFFF7E8, tmp;
	uint16 i = 0;

	len /= 2;
	if (len > 24)
	{
		len = 24;
	}

	while(len)
	{
		tmp = (chip_serial[len - 1] & 0xF0) >> 4;
		if (tmp > 9)
		{
			strSerial[i] = tmp + 'A' - 10;
		}
		else
		{
			strSerial[i] = tmp + '0';
		}
		len--;
		if (!len)
		{
			break;
		}

		tmp = (chip_serial[len - 1] & 0x0F) >> 0;
		if (tmp > 9)
		{
			strSerial[i + 2] = tmp + 'A' - 10;
		}
		else
		{
			strSerial[i + 2] = tmp + '0';
		}
		len--;

		chip_serial++;
		i += 4;
	}
}

void CDC_IF_Setup(uint32 baudrate, uint8 datatype, uint8 paritytype, uint8 stopbittype)
{
#if USB_CDC_TARGET_IF == USB_CDC_ON_USART
	static uint8 __if_inited = 0;
	USART_InitTypeDef USART_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

	if(!__if_inited)
	{
		__if_inited = 1;
		USART_DeInit(USART_DEF_PORT);
#if USART_AUX_PORT_EN
		USART_AUX_Port_Init();
#endif
		USART_Port_Init();
	}

	/* Enable the USART1 Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQChannel;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	USART_InitStructure.USART_BaudRate = baudrate;
	switch(stopbittype)
	{
	case 0:
		USART_InitStructure.USART_StopBits = USART_StopBits_1;
		break;
	case 1:
		USART_InitStructure.USART_StopBits = USART_StopBits_1_5;
		break;
	case 2:
		USART_InitStructure.USART_StopBits = USART_StopBits_2;
		break;
	default:
		USART_InitStructure.USART_StopBits = USART_StopBits_1;
		break;
	}
	switch(paritytype)
	{
	case 0:
		USART_InitStructure.USART_Parity = USART_Parity_No;
		USART_InitStructure.USART_WordLength = USART_WordLength_8b;
		break;
	case 1:
		USART_InitStructure.USART_Parity = USART_Parity_Odd;
		USART_InitStructure.USART_WordLength = USART_WordLength_9b;
		break;
	case 2:
		USART_InitStructure.USART_Parity = USART_Parity_Even;
		USART_InitStructure.USART_WordLength = USART_WordLength_9b;
		break;
	default:
		USART_InitStructure.USART_Parity = USART_Parity_No;
		USART_InitStructure.USART_WordLength = USART_WordLength_8b;
		break;
	}
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None; 

	USART_Init(USART_DEF_PORT, &USART_InitStructure);
	USART_ITConfig(USART_DEF_PORT, USART_IT_RXNE, ENABLE);
	USART_ITConfig(USART_DEF_PORT, USART_IT_TC, ENABLE);
	USART_Cmd(USART_DEF_PORT, ENABLE);
#endif

	GLOBAL_OUTPUT_Acquire();
	PWREXT_Acquire();
}

void CDC_IF_RX_Int(uint8 dat)
{
	uint8 buff[USB_DATA_SIZE];
	uint32 len;

	if(FIFO_Add_Byte(&CDC_IN_fifo, dat) != 1)
	{
		// OVERFLOW
		LED_RED_ON();
	}

	if(!CDC_USB_IsBusy)
	{
		CDC_USB_IsBusy = 1;
		len = FIFO_Get_Length(&CDC_IN_fifo);
		if(len > USB_DATA_SIZE)
		{
			len = USB_DATA_SIZE;
		}

		FIFO_Get_Buffer(&CDC_IN_fifo, buff, len);
		CDC_USB_Out(&buff[0], len);
	}
}

#if USB_CDC_TARGET_IF == USB_CDC_ON_USART
#	define CDC_IF_SendData(d)				USART_SendData(USART_DEF_PORT, (d))
#endif
void CDC_IF_TX_Int(void)
{
	if(FIFO_Get_Length(&CDC_OUT_fifo))
	{
		CDC_IF_SendData(FIFO_Get_Byte(&CDC_OUT_fifo));
	}
	else
	{
		CDC_USART_IsBusy = 0;
	}
}

s32 USB_Out(u8 *data, u32 len)
{
	u32 retry;

	buffer_in = data;
	usb_in_numofpackage = (len + USB_DATA_SIZE - 1) / USB_DATA_SIZE;
	if(!(len % USB_DATA_SIZE))
	{
		// 1 more package for ZLP
		usb_in_numofpackage++;
	}
	usb_in_data_remain = len;

#if USB_TX_DOUBLEBUFFER_EN
	if(GetENDPOINT(ENDP2) & EP_DTOG_RX)
	{
		if(usb_in_data_remain > USB_DATA_SIZE)
		{
			retry = USB_DATA_SIZE;
		}
		else
		{
			retry = usb_in_data_remain;
		}
		UserToPMABufferCopy(buffer_in, ENDP2_TXADDR1, retry);
		SetEPDblBuf1Count(ENDP2, EP_DBUF_IN, retry);
		FreeUserBuffer(ENDP2, EP_DBUF_IN);
		usb_in_data_remain -= retry;
		buffer_in += retry;

		if(usb_in_data_remain > 0)
		{
			if(usb_in_data_remain > USB_DATA_SIZE)
			{
				retry = USB_DATA_SIZE;
			}
			else
			{
				retry = usb_in_data_remain;
			}
			UserToPMABufferCopy(buffer_in, ENDP2_TXADDR0, retry);
			SetEPDblBuf0Count(ENDP2, EP_DBUF_IN, retry);
			usb_in_data_remain -= retry;
			buffer_in += retry;
		}
		else
		{
			SetEPDblBuf0Count(ENDP2, EP_DBUF_IN, 0);
		}
	}
	else
	{
		if(usb_in_data_remain > USB_DATA_SIZE)
		{
			retry = USB_DATA_SIZE;
		}
		else
		{
			retry = usb_in_data_remain;
		}
		UserToPMABufferCopy(buffer_in, ENDP2_TXADDR0, retry);
		SetEPDblBuf0Count(ENDP2, EP_DBUF_IN, retry);
		FreeUserBuffer(ENDP2, EP_DBUF_IN);
		usb_in_data_remain -= retry;
		buffer_in += retry;

		if(usb_in_data_remain > 0)
		{
			if(usb_in_data_remain > USB_DATA_SIZE)
			{
				retry = USB_DATA_SIZE;
			}
			else
			{
				retry = usb_in_data_remain;
			}
			UserToPMABufferCopy(buffer_in, ENDP2_TXADDR1, retry);
			SetEPDblBuf1Count(ENDP2, EP_DBUF_IN, retry);
			usb_in_data_remain -= retry;
			buffer_in += retry;
		}
		else
		{
			SetEPDblBuf1Count(ENDP2, EP_DBUF_IN, 0);
		}
	}
#else
	// start to send the first package
	if(usb_in_data_remain > USB_DATA_SIZE)
	{
		retry = USB_DATA_SIZE;
	}
	else
	{
		retry = usb_in_data_remain;
	}
	UserToPMABufferCopy(buffer_in, ENDP2_TXADDR, retry);
	SetEPTxCount(ENDP2, retry);
	usb_in_data_remain -= retry;
	buffer_in += retry;
#endif
	SetEPTxValid(ENDP2);

	retry = 0;
	while(usb_in_numofpackage)
	{
		if(retry++ > 0xFFFFF)
		{
			// timeout
			return -1;
		}
		if(usb_ovf)
		{
			// over flow(new command received, but old reply not sent)
			usb_ovf = 0;
			SetEPTxStatus(ENDP2, EP_TX_NAK);
			return -1;
		}
	}

	return len;
}

/*******************************************************************************
* Function Name	: USB_Cable_Config
* Description		: Software Connection/Disconnection of USB Cable
* Input					: None.
* Return				 : Status
*******************************************************************************/
void USB_Cable_Config (FunctionalState NewState)
{
}

/*******************************************************************************
* Function Name	: Enter_LowPowerMode
* Description		: Power-off system clocks and power while entering suspend mode
* Input					: None.
* Return				 : None.
*******************************************************************************/
void Enter_LowPowerMode(void)
{
}

/*******************************************************************************
* Function Name	: Leave_LowPowerMode
* Description		: Restores system clocks and power while exiting suspend mode
* Input					: None.
* Return				 : None.
*******************************************************************************/
void Leave_LowPowerMode(void)
{
}

#ifdef	DEBUG
/*******************************************************************************
* Function Name	: assert_failed
* Description		: Reports the name of the source file and the source line number
*									where the assert_param error has occurred.
* Input					: - file: pointer to the source file name
*									- line: assert_param error line source number
* Output				 : None
* Return				 : None
*******************************************************************************/
void assert_failed(u8* file, u32 line)
{ 
	/* User can add his own implementation to report the file name and line number,
		 ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

	/* Infinite loop */
	while (1)
	{
	}
}
#endif
