/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
* File Name					: usb_endp.c
* Author						 : MCD Application Team
* Version						: V1.0
* Date							 : 10/08/2007
* Description				: Endpoint routines
********************************************************************************
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "app_cfg.h"

#if USB_PROTOCOL == USB_ST_VCOM

#include "VSLLink.h"
#include "fifo.h"
#include "USB_proto.h"

#include "stm32f10x_lib.h"
#include "usb_lib.h"
#include "usb_desc.h"

#include "usb_bot.h"
#include "usb_istr.h"
#include "usb_cdc.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
u8 buffer_out[USB_DATA_BUFF_SIZE], *buffer_in = NULL;
vu32 count_out = 0;
vu32 usb_in_data_remain = 0, usb_in_numofpackage = 0;
vu32 buffer_ptr = 0;
vu32 usb_ovf = 0;

vu32 cmd_len = 0;
vu32 rep_len = 0;

// OUT means PC OUT, Device input
// IN means PC IN, Device output
#define CDC_OUT_buff			buffer_out
#define CDC_IN_buff				asyn_rx_buf

FIFO CDC_OUT_fifo = FIFO_Init(CDC_OUT_buff);
FIFO CDC_IN_fifo = FIFO_Init(CDC_IN_buff);

uint8 CDC_USART_IsBusy = 0;
uint8 CDC_USB_IsBusy = 0;

/* Private function prototypes -----------------------------------------------*/

/* Private functions ---------------------------------------------------------*/
/*******************************************************************************
* Function Name	: EP2_OUT_Callback
* Description		:
* Input					: None.
* Output				 : None.
* Return				 : None.
*******************************************************************************/
void EP3_OUT_Callback(void)
{
	u32 pkg_len;
	uint8 byte;
	uint8 buff[VIRTUAL_COM_PORT_DATA_SIZE];

	if(CDC_enable)
	{
#if USB_RX_DOUBLEBUFFER_EN
		if(GetENDPOINT(ENDP3) & EP_DTOG_TX)
		{
			FreeUserBuffer(ENDP3, EP_DBUF_OUT);
			pkg_len = GetEPDblBuf0Count(ENDP3);
			PMAToUserBufferCopy(buff, ENDP3_RXADDR0, pkg_len);
		}
		else
		{
			FreeUserBuffer(ENDP3, EP_DBUF_OUT);
			pkg_len = GetEPDblBuf1Count(ENDP3);
			PMAToUserBufferCopy(buff, ENDP3_RXADDR1, pkg_len);
		}
#else
		pkg_len = GetEPRxCount(ENDP3);
		PMAToUserBufferCopy(buff, ENDP3_RXADDR, pkg_len);
		SetEPRxValid(ENDP3);
#endif

		// copy to fifo
		if(FIFO_Add_Buffer(&CDC_OUT_fifo, buff, pkg_len) < pkg_len)
		{
			// OVERFLOW
			LED_RED_ON();
		}

		if(!CDC_USART_IsBusy)
		{
			CDC_USART_IsBusy = 1;
			// start to transmit first byte to USART
			byte = FIFO_Get_Byte(&CDC_OUT_fifo);
			CDC_USART_Out(byte);
		}
		return;
	}

	if(cmd_len & 0x80000000)
	{
		usb_ovf = 1;
		count_out = 0;
	}

#if USB_RX_DOUBLEBUFFER_EN
	if(GetENDPOINT(ENDP3) & EP_DTOG_TX)
	{
		FreeUserBuffer(ENDP3, EP_DBUF_OUT);
		pkg_len = GetEPDblBuf0Count(ENDP3);
		PMAToUserBufferCopy(buffer_out + count_out, ENDP3_RXADDR0, pkg_len);
	}
	else
	{
		FreeUserBuffer(ENDP3, EP_DBUF_OUT);
		pkg_len = GetEPDblBuf1Count(ENDP3);
		PMAToUserBufferCopy(buffer_out + count_out, ENDP3_RXADDR1, pkg_len);
	}
#else
	pkg_len = GetEPRxCount(ENDP3);
	PMAToUserBufferCopy(buffer_out + count_out, ENDP3_RXADDR, pkg_len);
	SetEPRxValid(ENDP3);
#endif

	if(!pkg_len)
	{
		return;
	}

	if(!count_out)
	{
		// first package
		if(buffer_out[0] <= VERSALOON_COMMON_CMD_END)
		{
			// Common Commands
			if(buffer_out[0] == VERSALOON_WRITE_OFFLINE_DATA)
			{
				cmd_len = buffer_out[1] + ((u16)buffer_out[2] << 8) + 7;
			}
			else
			{
				cmd_len = pkg_len;
			}
		}
#if USB_TO_XXX_EN
		else if(buffer_out[0] <= VERSALOON_USB_TO_XXX_CMD_END)
		{
			// USB_TO_XXX Support
			cmd_len = buffer_out[1] + ((u16)buffer_out[2] << 8);
		}
#endif
#if VSLLINK_EN
		else //if(buffer_out[0] <= VERSALOON_VSLLINK_CMD_END)
		{
			// JTAG Debugger Support
			if((buffer_out[0] == VSLLINK_CMD_HW_JTAGSEQCMD) || (buffer_out[0] == VSLLINK_CMD_HW_JTAGHLCMD) || (buffer_out[0] == VSLLINK_CMD_HW_JTAGRAWCMD))
			{
				cmd_len = buffer_out[1] + ((u16)buffer_out[2] << 8);
			}
			else
			{
				cmd_len = pkg_len;
			}
		}
#endif
	}
	count_out += pkg_len;

	// all data received?
	pkg_len = cmd_len;
	if(count_out >= pkg_len)
	{
		cmd_len |= 0x80000000;
	}
}

/*******************************************************************************
* Function Name	: EP2_IN_Callback
* Description		:
* Input					: None.
* Output				 : None.
* Return				 : None.
*******************************************************************************/
void EP2_IN_Callback(void)
{
	uint8 buff[VIRTUAL_COM_PORT_DATA_SIZE];
	uint32 len;

	if(CDC_enable)
	{
		len = FIFO_Get_Length(&CDC_IN_fifo);
		if(len > VIRTUAL_COM_PORT_DATA_SIZE)
		{
			len = VIRTUAL_COM_PORT_DATA_SIZE;
		}

		if(len)
		{
			CDC_USB_IsBusy = 1;
			len = FIFO_Get_Buffer(&CDC_IN_fifo, buff, len);
			CDC_USB_Out(&buff[0], len);
		}
		else
		{
			CDC_USB_IsBusy = 0;
		}
		return;
	}

	usb_in_numofpackage--;
#if USB_TX_DOUBLEBUFFER_EN
	if(GetENDPOINT(ENDP2) & EP_DTOG_RX)
	{
		if(usb_in_numofpackage > 0)
		{
			// enable next package
			FreeUserBuffer(ENDP2, EP_DBUF_IN);
		}
		if(usb_in_data_remain > 0)
		{
			if(usb_in_data_remain > VIRTUAL_COM_PORT_DATA_SIZE)
			{
				len = VIRTUAL_COM_PORT_DATA_SIZE;
			}
			else
			{
				len = usb_in_data_remain;
			}
			UserToPMABufferCopy(buffer_in, ENDP2_TXADDR0, len);
			SetEPDblBuf0Count(ENDP2, EP_DBUF_IN, len);
			usb_in_data_remain -= len;
			buffer_in += len;
		}
		else
		{
			SetEPDblBuf0Count(ENDP2, EP_DBUF_IN, 0);
		}
	}
	else
	{
		if(usb_in_numofpackage > 0)
		{
			// enable next package
			FreeUserBuffer(ENDP2, EP_DBUF_IN);
		}
		if(usb_in_data_remain > 0)
		{
			if(usb_in_data_remain > VIRTUAL_COM_PORT_DATA_SIZE)
			{
				len = VIRTUAL_COM_PORT_DATA_SIZE;
			}
			else
			{
				len = usb_in_data_remain;
			}
			UserToPMABufferCopy(buffer_in, ENDP2_TXADDR1, len);
			SetEPDblBuf1Count(ENDP2, EP_DBUF_IN, len);
			usb_in_data_remain -= len;
			buffer_in += len;
		}
		else
		{
			SetEPDblBuf1Count(ENDP2, EP_DBUF_IN, 0);
		}
	}
#else
	if(usb_in_data_remain > 0)
	{
		if(usb_in_data_remain > VIRTUAL_COM_PORT_DATA_SIZE)
		{
			len = VIRTUAL_COM_PORT_DATA_SIZE;
		}
		else
		{
			len = usb_in_data_remain;
		}
		UserToPMABufferCopy(buffer_in, ENDP2_TXADDR, len);
		SetEPTxCount(ENDP2, len);
		buffer_in += len;
		usb_in_data_remain -= len;
	}
	else
	{
		SetEPTxCount(ENDP2, 0);
	}
	if(usb_in_numofpackage > 0)
	{
		SetEPTxValid(ENDP2);
	}
#endif
}





#if USB_WITH_MASSSTORAGE
void EP4_OUT_Callback(void)
{
	Mass_Storage_Out();
}

void EP4_IN_Callback(void)
{
	Mass_Storage_In();
}
#endif

#endif  // #if USB_PROTOCOL == USB_ST_VCOM
/******************* (C) COPYRIGHT 2007 STMicroelectronics *****END OF FILE****/

