/**************************************************************************
 *  Copyright (C) 2008 by Simon Qian                                      *
 *  SimonQian@SimonQian.com                                               *
 *                                                                        *
 *  Project:    Versaloon                                                 *
 *  File:       MSP430_SBW.c                                              *
 *  Author:     SimonQian                                                 *
 *  Versaion:   See changelog                                             *
 *  Purpose:    MSP430_SBW interface implementation file                  *
 *  License:    See license                                               *
 *------------------------------------------------------------------------*
 *  Change Log:                                                           *
 *      YYYY-MM-DD:     What(by Who)                                      *
 *      2008-11-07:     created(by SimonQian)                             *
 **************************************************************************/

#include "app_cfg.h"
#include "MSP430_SBW.h"

#if INTERFACE_SBW_EN

#define MSP430_SBW_DELAY()				DelayUS(0)

void MSP430_SBW_Init(void)
{
	MSP430_SBWTDIO_SET();
	MSP430_SBWTDIO_SETOUTPUT();
	MSP430_SBWTCK_SET();
	MSP430_SBWTCK_SETOUTPUT();
	DelayMS(50);

	// enter 2-wire SBW mode
	// TEST(SBWTCK)  :        _______         _________
	//                _______|       |_______|         
	// /RST(SBWTDIO) :_________________________________
	//                                          |______
	{
		// Reset TEST logic
		MSP430_SBWTDIO_CLR();
		MSP430_SBWTCK_CLR();
		DelayMS(1);

		// entry sequence
		MSP430_SBWTDIO_SET();
		MSP430_SBWTCK_SET();
		MSP430_SBWTCK_CLR();
		MSP430_SBWTCK_SET();
	}
}

void MSP430_SBW_Fini(void)
{
	MSP430_SBWTDIO_SETINPUT();
	MSP430_SBWTCK_SETINPUT();
}

static uint8 MSP430_SBW_Clk(uint8 tms, uint8 tdi)
{
	uint8 tdo;

	// output tms
	if(tms > 0)
	{
		MSP430_SBWTDIO_SET();
	}
	else
	{
		MSP430_SBWTDIO_CLR();
	}
	MSP430_SBWTCK_CLR();
	MSP430_SBWTCK_SET();

	// output tdi
	if(tdi > 0)
	{
		MSP430_SBWTDIO_SET();
	}
	else
	{
		MSP430_SBWTDIO_CLR();
	}
	MSP430_SBWTCK_CLR();
	MSP430_SBWTCK_SET();

	// input tdo
	MSP430_SBWTDIO_SETINPUT();
	MSP430_SBWTCK_CLR();
	tdo = (MSP430_SBWTDIO_GET() > 0);
	MSP430_SBWTCK_SET();

	return tdo;
}

void MSP430_SBW_TCLK(uint8 tclk)
{
	if(tclk > 0)
	{
	}
	else
	{
	}
}

void MSP430_SBW_TCLK_STROKE(uint16 cnt)
{
}

static uint32 MSP430_SBW_Shift(uint32 data, uint8 len)
{
	return 0;
}

uint32 MSP430_SBW_DR(uint32 dr, uint8 len)
{
	return 0;
}

uint8 MSP430_SBW_IR(uint8 ir, uint8 len)
{
	return 0;
}

void MSP430_SBW_Reset(void)
{
}

uint8 MSP430_SBW_Poll(uint32 data, uint32 mask, uint32 value, uint8 len, uint16 poll_cnt)
{
	return 0;
}

#endif
