/**************************************************************************
 *  Copyright (C) 2008 by Simon Qian                                      *
 *  SimonQian@SimonQian.com                                               *
 *                                                                        *
 *  Project:    Versaloon                                                 *
 *  File:       app_cfg.h                                                 *
 *  Author:     SimonQian                                                 *
 *  Versaion:   See changelog                                             *
 *  Purpose:    configuration file                                        *
 *  License:    See license                                               *
 *------------------------------------------------------------------------*
 *  Change Log:                                                           *
 *      YYYY-MM-DD:     What(by Who)                                      *
 *      2008-11-07:     created(by SimonQian)                             *
 *      2008-11-22:     rewrite GPIO_Dir(by SimonQian)                    *
 **************************************************************************/

/*************************** Includes ***************************/
#include "stm32f10x_conf.h"
#include "stm32f10x_lib.h"
#include "app_type.h"
#include "port.h"
#include "myString.h"
#include "delay.h"
#include "HW.h"

/************************ What do U have ************************/
#define STM32_NANO						0x03
#define STM32_MINI						0x13
#define STM32_PRO						0x23
#define STM32_FULL						0x33

#define _HARDWARE_VER					STM32_PRO

#define _SYS_FREQUENCY					72		// in MHz

#define DELAY_TEST						0

/************************ What do U want ************************/
#define USB_AT_JTAGICE_MKII				0x00
#define USB_AT_DRAGON					0x01
#define USB_ST_VCOM						0x02
#define USB_PROTOCOL					USB_ST_VCOM

#define USB_WITH_NO_CDC					0				// 0 more USB Interface descriptor
#define USB_WITH_COMPOSITE_CDC			1				// 1 more USB Interface descriptor
#define USB_WITH_IAD_CDC				2				// 2 more USB Interface descriptors
#define USB_WITH_CDC					USB_WITH_IAD_CDC// only valid if not use ST_VCOM

#define USB_WITH_MASSSTORAGE			1

#if USB_WITH_CDC != 0
#	define USB_CDC_ON_USART				1

#	define USB_CDC_TARGET_IF			USB_CDC_ON_USART
#endif

#define AVR_ISP_EN						0
#define AVR_JTAG_EN						0
#define AVR_HVPP_EN						0
#define AVR_HVSP_EN						0
#define AVR_DW_EN						0
#define AVRX_JTAG_EN					0
#define AVRX_PDI_EN						0
#define AVR32_JTAG_EN					0

#define VSLLINK_EN						1
#define VSLLINK_HL_EN					1
#define USB_TO_XXX_EN					1

#define POWER_OUT_EN					1

// mass-product support
#define MP_EN							0
#if MP_EN
#	define MP_PSOC_EN					1
#	define MP_C8051F_C2_EN				0
#	define MP_C8051F_JTAG_EN			0
#	define MP_S5X_EN					0
#	define MP_STM8_EN					0
#	define MP_LPCICP_EN					0
#endif

#if USB_TO_XXX_EN
#	define USB_TO_USART_EN				0
#	define USB_TO_SPI_EN				0
#	define USB_TO_I2C_EN				0
#	define USB_TO_GPIO_EN				0
#	define USB_TO_CAN_EN				0
#	define USB_TO_PWM_EN				0
#	define USB_TO_ADC_EN				0
#	define USB_TO_DAC_EN				0
#	define USB_TO_POWER_EN				1
#	define USB_TO_JTAG_LL_EN			0
#	define USB_TO_JTAG_HL_EN			0
#	define USB_TO_ISSP_EN				0
#	define USB_TO_C2_EN					0
#	define USB_TO_MSP430_JTAG_EN		1
#	define USB_TO_MSP430_SBW_EN			0
#	define USB_TO_SBW_EN				0
#	define USB_TO_LPCICP_EN				0
#endif

#if (USB_PROTOCOL == USB_AT_JTAGICE_MKII) || (USB_PROTOCOL == USB_AT_DRAGON)
#	define AVR_EN						(AVR_ISP_EN | AVR_JTAG_EN | AVR_HVPP_EN | AVR_HVSP_EN)
#	define AVRX_EN						(AVRX_JTAG_EN | AVRX_PDI_EN)
#	define ATMEL_DEVICE_EN				(AVR_EN | AVRX_EN | AVR_DW_EN | AVR32_JTAG_EN)
#elif USB_PROTOCOL == USB_ST_VCOM
#	define AVR_EN						0
#	define AVRX_EN						0
#	define ATMEL_DEVICE_EN				0
#endif
#define MCU_PROG_EN						(ATMEL_DEVICE_EN)

/****************************** Mass ******************************/
#define Led_RW_ON()						
#define Led_RW_OFF()					


#define MSD_MEMORY_START_ADDR			0x0800A000	// from 40K
#define MSD_MEMORY_BLOCK_SIZE			1024

/****************************** FIFO for CDC ******************************/
#define FIFO_DISABLE_READ_INT()			Disable_CDC_Int()
#define FIFO_RESTORE_READ_INT()			Enable_CDC_Int()
#define FIFO_DISABLE_WRITE_INT()		Disable_CDC_Int()
#define FIFO_RESTORE_WRITE_INT()		Enable_CDC_Int()

/****************************** Power ******************************/
#define PWREXT_EN_PORT					GPIOA
#define PWREXT_EN_PIN					GPIO_PIN_14

#define PWREXT_INIT()					PWREXT_DISABLE()
#define PWREXT_ENABLE()					do{\
											GPIO_ClrPins(PWREXT_EN_PORT, GPIO_PIN_GetMask(PWREXT_EN_PIN));\
											GPIO_Dir(PWREXT_EN_PORT, GPIO_MODE_OUT_PP, PWREXT_EN_PIN);\
										}while(0)
#define PWREXT_DISABLE()				GPIO_Dir(PWREXT_EN_PORT, GPIO_MODE_IN_FLOATING, PWREXT_EN_PIN)

/****************************** Global Output ******************************/
#define GLOBAL_OUTPUT_ENABLE_PORT		GPIOB
#define GLOBAL_OUTPUT_ENABLE_PIN		GPIO_PIN_1

#if (_HARDWARE_VER == STM32_PRO)
#	define GLOBAL_OUTPUT_INIT()			do{\
											GLOBAL_OUTPUT_DISABLE();\
											GPIO_Dir(GLOBAL_OUTPUT_ENABLE_PORT, GPIO_MODE_OUT_PP, GLOBAL_OUTPUT_ENABLE_PIN);\
										}while(0)
#	define GLOBAL_OUTPUT_ENABLE()		GPIO_ClrPins(GLOBAL_OUTPUT_ENABLE_PORT, GPIO_PIN_GetMask(GLOBAL_OUTPUT_ENABLE_PIN))
#	define GLOBAL_OUTPUT_DISABLE()		GPIO_SetPins(GLOBAL_OUTPUT_ENABLE_PORT, GPIO_PIN_GetMask(GLOBAL_OUTPUT_ENABLE_PIN))
#else
#	define GLOBAL_OUTPUT_INIT()			
#	define GLOBAL_OUTPUT_ENABLE()		
#	define GLOBAL_OUTPUT_DISABLE()		
#endif

/****************************** SW ******************************/
#define SW_RST_PORT						JTAG_TAP_TRST_PORT
#define SW_RST_PIN						JTAG_TAP_TRST_PIN
#define SW_PORT							JTAG_TAP_SRST_PORT
#define SW_PIN							JTAG_TAP_SRST_PIN
#define SW_DIR_PORT						GPIOB
#define SW_DIR_PIN						GPIO_PIN_12

#define SW_DIR_INIT()					do{\
											SW_SETINPUT();\
											GPIO_Dir(SW_DIR_PORT, GPIO_MODE_OUT_PP, SW_DIR_PIN);\
										}while(0)
#define SW_SETINPUT()					do{\
											GPIO_Dir(SW_PORT, GPIO_MODE_IN_FLOATING, SW_PIN);\
											GPIO_ClrPins(SW_DIR_PORT, GPIO_PIN_GetMask(SW_DIR_PIN));\
										}while(0)
#define SW_SETOUTPUT()					do{\
											GPIO_SetPins(SW_DIR_PORT, GPIO_PIN_GetMask(SW_DIR_PIN));\
											GPIO_Dir(SW_PORT, GPIO_MODE_OUT_PP, SW_PIN);\
										}while(0)
#define SW_SET()						GPIO_SetPins(SW_PORT, GPIO_PIN_GetMask(SW_PIN))
#define SW_CLR()						GPIO_ClrPins(SW_PORT, GPIO_PIN_GetMask(SW_PIN))
#define SW_GET()						GPIO_GetInPins(SW_PORT, GPIO_PIN_GetMask(SW_PIN))

#define SW_RST_SETOUTPUT()				GPIO_Dir(SW_RST_PORT, GPIO_MODE_OUT_PP, SW_RST_PIN)
#define SW_RST_SETINPUT()				GPIO_Dir(SW_RST_PORT, GPIO_MODE_IN_FLOATING, SW_RST_PIN)
#define SW_RST_SET()					GPIO_SetPins(SW_RST_PORT, GPIO_PIN_GetMask(SW_RST_PIN))
#define SW_RST_CLR()					GPIO_ClrPins(SW_RST_PORT, GPIO_PIN_GetMask(SW_RST_PIN))

/***************************** JTAG ******************************/
#define JTAG_TAP_PORT					GPIOB
#define JTAG_TAP_TCK_PIN				GPIO_PIN_13
#define JTAG_TAP_TDO_PIN				GPIO_PIN_14
#define JTAG_TAP_TDI_PIN				GPIO_PIN_15
#define JTAG_TAP_TMS_PORT				GPIOB
#define JTAG_TAP_TMS_PIN				GPIO_PIN_4
#define JTAG_TAP_RTCK_PORT				GPIOA
#define JTAG_TAP_RTCK_PIN				GPIO_PIN_8

#if (_HARDWARE_VER == STM32_PRO)
#	define JTAG_TAP_SRST_PORT			GPIOB
#	define JTAG_TAP_SRST_PIN			GPIO_PIN_11
#	define JTAG_TAP_TRST_PORT			GPIOB
#	define JTAG_TAP_TRST_PIN			GPIO_PIN_10
#	define JTAG_HAS_USER_PIN			1
#elif (_HARDWARE_VER == STM32_MINI)
#	define JTAG_TAP_SRST_PORT			GPIOB
#	define JTAG_TAP_SRST_PIN			GPIO_PIN_10
#	define JTAG_TAP_TRST_PORT			GPIOB
#	define JTAG_TAP_TRST_PIN			GPIO_PIN_11
#	define JTAG_HAS_USER_PIN			1
#elif (_HARDWARE_VER == STM32_NANO)
#	define JTAG_TAP_SRST_PORT			GPIOA
#	define JTAG_TAP_SRST_PIN			GPIO_PIN_10
#	define JTAG_TAP_TRST_PORT			GPIOA
#	define JTAG_TAP_TRST_PIN			GPIO_PIN_9
#	define JTAG_HAS_USER_PIN			0
#endif

#if JTAG_HAS_USER_PIN
#	define JTAG_TAP_USR_PORT			GPIOA
#	define JTAG_TAP_USR1_PIN			GPIO_PIN_9
#	define JTAG_TAP_USR2_PIN			GPIO_PIN_10

#	define JTAG_TAP_USR1_SETOUTPUT()	GPIO_Dir(JTAG_TAP_USR_PORT, GPIO_MODE_OUT_PP, JTAG_TAP_USR1_PIN)
#	define JTAG_TAP_USR1_SETINPUT()		GPIO_Dir(JTAG_TAP_USR_PORT, GPIO_MODE_IN_FLOATING, JTAG_TAP_USR1_PIN)
#	define JTAG_TAP_USR1_SET()			GPIO_SetPins(JTAG_TAP_USR_PORT, GPIO_PIN_GetMask(JTAG_TAP_USR1_PIN))
#	define JTAG_TAP_USR1_CLR()			GPIO_ClrPins(JTAG_TAP_USR_PORT, GPIO_PIN_GetMask(JTAG_TAP_USR1_PIN))
#	define JTAG_TAP_USR1_GET()			GPIO_GetOutPins(JTAG_TAP_USR_PORT, GPIO_PIN_GetMask(JTAG_TAP_USR1_PIN))

#	define JTAG_TAP_USR2_SETINPUT()		GPIO_Dir(JTAG_TAP_USR_PORT, GPIO_MODE_IN_FLOATING, JTAG_TAP_USR2_PIN)
#	define JTAG_TAP_USR2_GET()			GPIO_GetInPins(JTAG_TAP_USR_PORT, GPIO_PIN_GetMask(JTAG_TAP_USR2_PIN))
#endif

#define JTAG_TAP_TMS_SETOUTPUT()		GPIO_Dir(JTAG_TAP_TMS_PORT, GPIO_MODE_OUT_PP, JTAG_TAP_TMS_PIN)
#define JTAG_TAP_TMS_SETINPUT()			GPIO_Dir(JTAG_TAP_TMS_PORT, GPIO_MODE_IN_FLOATING, JTAG_TAP_TMS_PIN)
#define JTAG_TAP_TMS_SET()				GPIO_SetPins(JTAG_TAP_PORT, GPIO_PIN_GetMask(JTAG_TAP_TMS_PIN))
#define JTAG_TAP_TMS_CLR()				GPIO_ClrPins(JTAG_TAP_PORT, GPIO_PIN_GetMask(JTAG_TAP_TMS_PIN))
#define JTAG_TAP_TMS_GET()				GPIO_GetOutPins(JTAG_TAP_PORT, GPIO_PIN_GetMask(JTAG_TAP_TMS_PIN))

#define JTAG_TAP_TCK_SETOUTPUT()		GPIO_Dir(JTAG_TAP_PORT, GPIO_MODE_OUT_PP, JTAG_TAP_TCK_PIN)
#define JTAG_TAP_TCK_SETINPUT()			GPIO_Dir(JTAG_TAP_PORT, GPIO_MODE_IN_FLOATING, JTAG_TAP_TCK_PIN)
#define JTAG_TAP_TCK_SET()				GPIO_SetPins(JTAG_TAP_PORT, GPIO_PIN_GetMask(JTAG_TAP_TCK_PIN))
#define JTAG_TAP_TCK_CLR()				GPIO_ClrPins(JTAG_TAP_PORT, GPIO_PIN_GetMask(JTAG_TAP_TCK_PIN))
#define JTAG_TAP_TCK_GET()				GPIO_GetOutPins(JTAG_TAP_PORT, GPIO_PIN_GetMask(JTAG_TAP_TCK_PIN))

#define JTAG_TAP_TDI_SETOUTPUT()		GPIO_Dir(JTAG_TAP_PORT, GPIO_MODE_OUT_PP, JTAG_TAP_TDI_PIN)
#define JTAG_TAP_TDI_SETINPUT()			GPIO_Dir(JTAG_TAP_PORT, GPIO_MODE_IN_FLOATING, JTAG_TAP_TDI_PIN)
#define JTAG_TAP_TDI_SET()				GPIO_SetPins(JTAG_TAP_PORT, GPIO_PIN_GetMask(JTAG_TAP_TDI_PIN))
#define JTAG_TAP_TDI_CLR()				GPIO_ClrPins(JTAG_TAP_PORT, GPIO_PIN_GetMask(JTAG_TAP_TDI_PIN))
#define JTAG_TAP_TDI_GET()				GPIO_GetOutPins(JTAG_TAP_PORT, GPIO_PIN_GetMask(JTAG_TAP_TDI_PIN))

#define JTAG_TAP_TDO_SETINPUT()			GPIO_Dir(JTAG_TAP_PORT, GPIO_MODE_IN_FLOATING, JTAG_TAP_TDO_PIN)
#define JTAG_TAP_TDO_SET()				GPIO_SetPins(JTAG_TAP_PORT, GPIO_PIN_GetMask(JTAG_TAP_TDO_PIN))
#define JTAG_TAP_TDO_CLR()				GPIO_ClrPins(JTAG_TAP_PORT, GPIO_PIN_GetMask(JTAG_TAP_TDO_PIN))
#define JTAG_TAP_TDO_GET()				GPIO_GetInPins(JTAG_TAP_PORT, GPIO_PIN_GetMask(JTAG_TAP_TDO_PIN))

#define JTAG_TAP_TRST_SETOUTPUT()		GPIO_Dir(JTAG_TAP_TRST_PORT, GPIO_MODE_OUT_PP, JTAG_TAP_TRST_PIN)
#define JTAG_TAP_TRST_SETINPUT()		GPIO_Dir(JTAG_TAP_TRST_PORT, GPIO_MODE_IN_FLOATING, JTAG_TAP_TRST_PIN)
#define JTAG_TAP_TRST_SET()				GPIO_SetPins(JTAG_TAP_TRST_PORT, GPIO_PIN_GetMask(JTAG_TAP_TRST_PIN))
#define JTAG_TAP_TRST_CLR()				GPIO_ClrPins(JTAG_TAP_TRST_PORT, GPIO_PIN_GetMask(JTAG_TAP_TRST_PIN))
#define JTAG_TAP_TRST_GET()				GPIO_GetOutPins(JTAG_TAP_TRST_PORT, GPIO_PIN_GetMask(JTAG_TAP_TRST_PIN))

#define JTAG_TAP_RTCK_SETINPUT()		GPIO_Dir(JTAG_TAP_RTCK_PORT, GPIO_MODE_IN_FLOATING, JTAG_TAP_RTCK_PIN)
#define JTAG_TAP_RTCK_GET()				GPIO_GetInPins(JTAG_TAP_RTCK_PORT, GPIO_PIN_GetMask(JTAG_TAP_RTCK_PIN))

#define JTAG_TAP_SRST_SETOUTPUT()		SW_SETOUTPUT()
#define JTAG_TAP_SRST_SETINPUT()		SW_SETINPUT()
#define JTAG_TAP_SRST_SET()				SW_SET()
#define JTAG_TAP_SRST_CLR()				SW_CLR()
#define JTAG_TAP_SRST_GET()				SW_GET()


#define JTAG_TAP_HS_SPI_M				SPI2
#define JTAG_TAP_HS_SPI_S				SPI1

// DMA
#define JTAG_TAP_HS_SPI_M_TX_DMA		DMA1_Channel5
#define JTAG_TAP_HS_SPI_M_RX_DMA		DMA1_Channel4
#define JTAG_TAP_HS_SPI_S_TX_DMA		DMA1_Channel3

#define JTAG_TAP_HS_SPI_EnableDMA()		do{\
											SPI_I2S_DMACmd(JTAG_TAP_HS_SPI_M, SPI_I2S_DMAReq_Rx, ENABLE);\
											SPI_I2S_DMACmd(JTAG_TAP_HS_SPI_S, SPI_I2S_DMAReq_Tx, ENABLE);\
										}while(0)

#define JTAG_TAP_HS_SPI_M_RX_DMA_LEN(l)	(JTAG_TAP_HS_SPI_M_RX_DMA->CNDTR = (uint32)(l))
#define JTAG_TAP_HS_SPI_M_RX_DMA_ADDR(a)(JTAG_TAP_HS_SPI_M_RX_DMA->CMAR = (uint32)(a))
#define JTAG_TAP_HS_SPI_M_RX_DMA_EN()	(JTAG_TAP_HS_SPI_M_RX_DMA->CCR |= (uint32)1)
#define JTAG_TAP_HS_SPI_M_RX_DMA_DIS()	(JTAG_TAP_HS_SPI_M_RX_DMA->CCR &= ~(uint32)1)
#define JTAG_TAP_HS_SPI_M_RX_DMA_WAIT()	do{while(!(DMA1->ISR & DMA1_FLAG_TC4)); DMA1->IFCR = DMA1_FLAG_TC4;}while(0)

#define JTAG_TAP_HS_SPI_S_TX_DMA_LEN(l)	(JTAG_TAP_HS_SPI_S_TX_DMA->CNDTR = (uint32)(l))
#define JTAG_TAP_HS_SPI_S_TX_DMA_ADDR(a)(JTAG_TAP_HS_SPI_S_TX_DMA->CMAR = (uint32)(a))
#define JTAG_TAP_HS_SPI_S_TX_DMA_EN()	(JTAG_TAP_HS_SPI_S_TX_DMA->CCR |= (uint32)1)
#define JTAG_TAP_HS_SPI_S_TX_DMA_DIS()	(JTAG_TAP_HS_SPI_S_TX_DMA->CCR &= ~(uint32)1)
#define JTAG_TAP_HS_SPI_S_TX_DMA_WAIT()	do{while(!(DMA1->ISR & DMA1_FLAG_TC3)); DMA1->IFCR = DMA1_FLAG_TC3;}while(0)

#define JTAG_TAP_HS_MPORT				GPIOB

#define JTAG_TAP_HS_SPORT				GPIOB
#define JTAG_TAP_TCK1_PIN				GPIO_PIN_3
#define JTAG_TAP_TMS_PIN				GPIO_PIN_4

#define JTAG_TAP_HS_PortInit()			do{\
											GPIO_Dir(JTAG_TAP_HS_MPORT, GPIO_MODE_AF_PP, JTAG_TAP_TCK_PIN);\
											GPIO_Dir(JTAG_TAP_HS_MPORT, GPIO_MODE_AF_PP, JTAG_TAP_TDO_PIN);\
											GPIO_Dir(JTAG_TAP_HS_MPORT, GPIO_MODE_AF_PP, JTAG_TAP_TDI_PIN);\
											GPIO_Dir(JTAG_TAP_HS_SPORT, GPIO_MODE_AF_PP, JTAG_TAP_TCK1_PIN);\
											GPIO_Dir(JTAG_TAP_HS_SPORT, GPIO_MODE_AF_PP, JTAG_TAP_TMS_PIN);\
										}while(0)
#define JTAG_TAP_HS_PortFini()			do{\
											GPIO_Dir(JTAG_TAP_HS_MPORT, GPIO_MODE_IN_FLOATING, JTAG_TAP_TCK_PIN);\
											GPIO_Dir(JTAG_TAP_HS_MPORT, GPIO_MODE_IN_FLOATING, JTAG_TAP_TDO_PIN);\
											GPIO_Dir(JTAG_TAP_HS_MPORT, GPIO_MODE_IN_FLOATING, JTAG_TAP_TDI_PIN);\
											GPIO_Dir(JTAG_TAP_HS_SPORT, GPIO_MODE_IN_FLOATING, JTAG_TAP_TCK1_PIN);\
											GPIO_Dir(JTAG_TAP_HS_SPORT, GPIO_MODE_IN_FLOATING, JTAG_TAP_TMS_PIN);\
										}while(0)

#define JTAG_TAP_HS_SetSpeed(div)		SPI_Configuration(JTAG_TAP_HS_SPI_M,SPI_Mode_Master,(div),\
														SPI_FirstBit_LSB,SPI_CPOL_High,SPI_CPHA_2Edge)

#define JTAG_TAP_HS_TMS_Out(tms)		JTAG_TAP_HS_SPI_S->DR = (tms)
#define JTAG_TAP_HS_TDI_Out(tdi)		JTAG_TAP_HS_SPI_M->DR = (tdi)
#define JTAG_TAP_HS_Out(tms, tdi)		do{JTAG_TAP_HS_TMS_Out(tms); JTAG_TAP_HS_TDI_Out(tdi);}while(0)
#define JTAG_TAP_HS_In()				JTAG_TAP_HS_SPI_M->DR

#define JTAG_TAP_HS_WaitTxReady()		while(!(JTAG_TAP_HS_SPI_M->SR & SPI_I2S_FLAG_TXE))
#define JTAG_TAP_HS_WaitRxReady()		while(!(JTAG_TAP_HS_SPI_M->SR & SPI_I2S_FLAG_RXNE))
#define JTAG_TAP_HS_WaitReady()			while(JTAG_TAP_HS_SPI_M->SR & SPI_I2S_FLAG_BSY)

/****************************** SPI ******************************/
#define SPI_Interface					SPI2
#define SPI_GPIO_PORT					JTAG_TAP_PORT
#define SPI_MOSI_PIN					JTAG_TAP_TDI_PIN
#define SPI_MISO_PIN					JTAG_TAP_TDO_PIN
#define SPI_SCK_PIN						JTAG_TAP_TCK_PIN

#define SPI_Conf(br)					SPI_Configuration(SPI_Interface,SPI_Mode_Master,\
														(br),SPI_FirstBit_MSB,\
														SPI_CPOL_Low,SPI_CPHA_1Edge)
#define SPI_SetClkHw(br)				do{SPI_InitStructure.SPI_BaudRatePrescaler = (br);\
										SPI_Init(SPI_Interface, &SPI_InitStructure);}while(0)

#define SPI_SetMOSI()					GPIO_SetPins(SPI_GPIO_PORT, GPIO_PIN_GetMask(SPI_MOSI_PIN))
#define SPI_ClrMOSI()					GPIO_ClrPins(SPI_GPIO_PORT, GPIO_PIN_GetMask(SPI_MOSI_PIN))
#define SPI_GetMOSI()					GPIO_GetOutPins(SPI_GPIO_PORT, GPIO_PIN_GetMask(SPI_MOSI_PIN))

#define SPI_GetMISO()					GPIO_GetInPins(SPI_GPIO_PORT, GPIO_PIN_GetMask(SPI_MISO_PIN))

#define SPI_SetSCK()					GPIO_SetPins(SPI_GPIO_PORT, GPIO_PIN_GetMask(SPI_SCK_PIN))
#define SPI_ClrSCK()					GPIO_ClrPins(SPI_GPIO_PORT, GPIO_PIN_GetMask(SPI_SCK_PIN))
#define SPI_GetSCK()					GPIO_GetOutPins(SPI_GPIO_PORT, GPIO_PIN_GetMask(SPI_SCK_PIN))

#define SPI_Disable()					SPI_I2S_DeInit(SPI_Interface)
#define SPI_SetData(d)					SPI_I2S_SendData(SPI_Interface, (d))
#define SPI_GetData()					SPI_I2S_ReceiveData(SPI_Interface)
#define SPI_WaitReady()					SPI_WaitRxReady()//while((SPI_Interface->SR & SPI_I2S_FLAG_BSY) == (u16)SET)
#define SPI_WaitRxReady()				while((SPI_Interface->SR & SPI_I2S_FLAG_RXNE) == (u16)RESET)
#define SPI_WaitTxReady()				while((SPI_Interface->SR & SPI_I2S_FLAG_TXE) == (u16)RESET)

#define SPI_AllInput()					do{\
											GLOBAL_OUTPUT_Release();\
											GPIO_Dir(SPI_GPIO_PORT, GPIO_MODE_IN_FLOATING, SPI_SCK_PIN);\
											GPIO_Dir(SPI_GPIO_PORT, GPIO_MODE_IN_FLOATING, SPI_MISO_PIN);\
											GPIO_Dir(SPI_GPIO_PORT, GPIO_MODE_IN_FLOATING, SPI_MOSI_PIN);\
										}while(0)
#define SPI_AllSPIIO()					do{\
											GLOBAL_OUTPUT_Acquire();\
											GPIO_Dir(SPI_GPIO_PORT, GPIO_MODE_OUT_PP, SPI_SCK_PIN);\
											GPIO_Dir(SPI_GPIO_PORT, GPIO_MODE_OUT_PP, SPI_MOSI_PIN);\
											GPIO_Dir(SPI_GPIO_PORT, GPIO_MODE_IPU, SPI_MISO_PIN);\
										}while(0)
#define SPI_AllSPIHW()					do{\
											GLOBAL_OUTPUT_Acquire();\
											GPIO_Dir(SPI_GPIO_PORT, GPIO_MODE_AF_PP, SPI_SCK_PIN);\
											GPIO_Dir(SPI_GPIO_PORT, GPIO_MODE_AF_PP, SPI_MISO_PIN);\
											GPIO_Dir(SPI_GPIO_PORT, GPIO_MODE_AF_PP, SPI_MOSI_PIN);\
										}while(0)


/****************************** ISP ******************************/
#define ISP_SetRST()					JTAG_TAP_SRST_SET()
#define ISP_ClrRST()					JTAG_TAP_SRST_CLR()
#define ISP_GetRST()					JTAG_TAP_SRST_GET()

#define ISP_SetRSTOutput()				SW_SETOUTPUT()
#define ISP_SetRSTInput()				SW_SETINPUT()

#define ISP_WaitReady()					while(!(SPI_GetFlagStatus(SPI_Interface,SPI_FLAG_RXNE) == SET))

/************************** MSP430 JTAG ****************************/
#define MSP430_JTAG_TEST_SETOUTPUT()	JTAG_TAP_TRST_SETOUTPUT()
#define MSP430_JTAG_TEST_SETINPUT()		JTAG_TAP_TRST_SETINPUT()
#define MSP430_JTAG_TEST_SET()			JTAG_TAP_TRST_SET()
#define MSP430_JTAG_TEST_CLR()			JTAG_TAP_TRST_CLR()
#define MSP430_JTAG_TEST_GET()			JTAG_TAP_TRST_GET()

/************************** MSP430 SBW ****************************/
#define MSP430_SBWTDIO_SETOUTPUT()		SW_SETOUTPUT()
#define MSP430_SBWTDIO_SETINPUT()		SW_SETINPUT()
#define MSP430_SBWTDIO_SET()			SW_SET()
#define MSP430_SBWTDIO_CLR()			SW_CLR()
#define MSP430_SBWTDIO_GET()			SW_GET()

#define MSP430_SBWTCK_SETOUTPUT()		MSP430_JTAG_TCK_SETOUTPUT()
#define MSP430_SBWTCK_SETINPUT()		MSP430_JTAG_TCK_SETINPUT()
#define MSP430_SBWTCK_SET()				MSP430_JTAG_TCK_SET()
#define MSP430_SBWTCK_CLR()				MSP430_JTAG_TCK_CLR()

/************************** PSoC ISSP ****************************/
#define ISSP_PORT						JTAG_TAP_PORT
#define ISSP_SCLK_PIN					JTAG_TAP_TCK_PIN

#define ISSP_SetSDATA()					SW_SET()
#define ISSP_ClrSDATA()					SW_CLR()
#define ISSP_GetSDATA()					SW_GET()
#define ISSP_SetSDATAOutput()			SW_SETOUTPUT()
#define ISSP_SetSDATAInput()			SW_SETINPUT()

#define ISSP_SetSCLK()					GPIO_SetPins(ISSP_PORT, GPIO_PIN_GetMask(ISSP_SCLK_PIN))
#define ISSP_ClrSCLK()					GPIO_ClrPins(ISSP_PORT, GPIO_PIN_GetMask(ISSP_SCLK_PIN))
#define ISSP_SetSCLKOutput()			GPIO_Dir(ISSP_PORT, GPIO_MODE_OUT_PP, ISSP_SCLK_PIN)
#define ISSP_SetSCLKInput()				GPIO_Dir(ISSP_PORT, GPIO_MODE_IN_FLOATING, ISSP_SCLK_PIN)

#define ISSP_SetXRES()					SW_RST_SET()
#define ISSP_ClrXRES()					SW_RST_CLR()
#define ISSP_SetXRESOutput()			SW_RST_SETOUTPUT()
#define ISSP_SetXRESInput()				SW_RST_SETINPUT()

#define ISSP_PowerOn()					PWREXT_ENABLE()
#define ISSP_PowerOff()					PWREXT_DISABLE()

/****************************** C2 ******************************/
#define C2_PORT							JTAG_TAP_PORT
#define C2_C2CK_PIN						JTAG_TAP_TCK_PIN

#define C2_C2CK_SET()					GPIO_SetPins(C2_PORT, GPIO_PIN_GetMask(C2_C2CK_PIN))
#define C2_C2CK_CLR()					GPIO_ClrPins(C2_PORT, GPIO_PIN_GetMask(C2_C2CK_PIN))
#define C2_C2CK_SETOUTPUT()				GPIO_Dir(C2_PORT, GPIO_MODE_OUT_PP, C2_C2CK_PIN)
#define C2_C2CK_SETINPUT()				GPIO_Dir(C2_PORT, GPIO_MODE_IN_FLOATING, C2_C2CK_PIN)

#define C2_C2D_SET()					SW_SET()
#define C2_C2D_CLR()					SW_CLR()
#define C2_C2D_GET()					SW_GET()
#define C2_C2D_SETOUTPUT()				SW_SETOUTPUT()
#define C2_C2D_SETINPUT()				SW_SETINPUT()

/****************************** LPCICP ******************************/
#define LPCICP_PORT						JTAG_TAP_PORT
#define LPCICP_SCLK_PIN					JTAG_TAP_TCK_PIN

#define LPCICP_SetSDATA()				SW_SET()
#define LPCICP_ClrSDATA()				SW_CLR()
#define LPCICP_GetSDATA()				SW_GET()
#define LPCICP_SetSDATAOutput()			SW_SETOUTPUT()
#define LPCICP_SetSDATAInput()			SW_SETINPUT()

#define LPCICP_SetSCLK()				GPIO_SetPins(LPCICP_PORT, GPIO_PIN_GetMask(LPCICP_SCLK_PIN))
#define LPCICP_ClrSCLK()				GPIO_ClrPins(LPCICP_PORT, GPIO_PIN_GetMask(LPCICP_SCLK_PIN))
#define LPCICP_SetSCLKOutput()			GPIO_Dir(LPCICP_PORT, GPIO_MODE_OUT_PP, LPCICP_SCLK_PIN)
#define LPCICP_SetSCLKInput()			GPIO_Dir(LPCICP_PORT, GPIO_MODE_IN_FLOATING, LPCICP_SCLK_PIN)

#define LPCICP_SetXRES()				SW_RST_SET()
#define LPCICP_ClrXRES()				SW_RST_CLR()
#define LPCICP_SetXRESOutput()			SW_RST_SETOUTPUT()
#define LPCICP_SetXRESInput()			SW_RST_SETINPUT()
/*
#define LPCICP_PowerOn()				PWREXT_ENABLE()
#define LPCICP_PowerOff()				PWREXT_DISABLE()
*/
/****************************** USART ******************************/
// to change USART port below, modify the interrupt handler
#define USART_DEF_PORT					USART1
#define USART_AUX_PORT_EN				0

#define USART_PORT						GPIOA
#define USART_TXD_PIN					GPIO_PIN_9
#define USART_RXD_PIN					GPIO_PIN_10

#define USART_AUX_PORT					GPIOB
#define USART_DTR_PIN					GPIO_PIN_10
#define USART_RTS_PIN					GPIO_PIN_11
#define USART_DSR_PIN					
#define USART_CTS_PIN					
#define USART_RI_PIN					

#define USART_DTR_SET()					GPIO_SetPins(USART_AUX_PORT, GPIO_PIN_GetMask(USART_DTR_PIN))
#define USART_DTR_CLR()					GPIO_ClrPins(USART_AUX_PORT, GPIO_PIN_GetMask(USART_DTR_PIN))
#define USART_RTS_SET()					GPIO_SetPins(USART_AUX_PORT, GPIO_PIN_GetMask(USART_RTS_PIN))
#define USART_RTS_CLR()					GPIO_ClrPins(USART_AUX_PORT, GPIO_PIN_GetMask(USART_RTS_PIN))

#define USART_AUX_Port_Init()			do{\
											GPIO_Dir(USART_AUX_PORT, GPIO_MODE_OUT_PP, USART_DTR_PIN);\
											GPIO_Dir(USART_AUX_PORT, GPIO_MODE_OUT_PP, USART_RTS_PIN);\
										}while(0)
#define USART_Port_Init()				do{\
											GPIO_Dir(USART_PORT, GPIO_MODE_AF_PP, USART_TXD_PIN);\
											GPIO_Dir(USART_PORT, GPIO_MODE_IN_FLOATING, USART_RXD_PIN);\
										}while(0)

/****************************** ADC ******************************/
#define TVCC_PORT						GPIOB
#define TVCC_PIN						GPIO_PIN_0
#define TVCC_ADC_CHANNEL				ADC_Channel_8

#define TVCC_SAMPLE_MIN_POWER			1800

#if _HARDWARE_VER == STM32_MINI
#	define TVCC_SAMPLE_DIV				6
#else
#	define TVCC_SAMPLE_DIV				2
#endif
#define TVCC_SAMPLE_VREF				3300
#define TVCC_SAMPLE_MAXVAL				4096

/****************************** LED ******************************/
#if (_HARDWARE_VER == STM32_MINI) || (_HARDWARE_VER == STM32_PRO)
#	define LED_PORT						GPIOB
#	define LED_RED_PIN					GPIO_PIN_6
#	define LED_GREEN_PIN				GPIO_PIN_7
#	define LED_RED_ON()					GPIO_ClrPins(LED_PORT, GPIO_PIN_GetMask(LED_RED_PIN))
#	define LED_RED_OFF()				GPIO_SetPins(LED_PORT, GPIO_PIN_GetMask(LED_RED_PIN))
#	define LED_GREEN_ON()				GPIO_ClrPins(LED_PORT, GPIO_PIN_GetMask(LED_GREEN_PIN))
#	define LED_GREEN_OFF()				GPIO_SetPins(LED_PORT, GPIO_PIN_GetMask(LED_GREEN_PIN))
#	define LED_ALL_ON()					GPIO_ClrPins(LED_PORT, GPIO_PIN_GetMask(LED_GREEN_PIN) | GPIO_PIN_GetMask(LED_RED_PIN))
#	define LED_ALL_OFF()				GPIO_SetPins(LED_PORT, GPIO_PIN_GetMask(LED_GREEN_PIN) | GPIO_PIN_GetMask(LED_RED_PIN))
#elif (_HARDWARE_VER == STM32_NANO)
#	define LED_PORT						GPIOA
#	define LED_RED_PIN					GPIO_PIN_1
#	define LED_GREEN_PIN				GPIO_PIN_2
#	define LED_RED_ON()					GPIO_SetPins(LED_PORT, GPIO_PIN_GetMask(LED_RED_PIN))
#	define LED_RED_OFF()				GPIO_ClrPins(LED_PORT, GPIO_PIN_GetMask(LED_RED_PIN))
#	define LED_GREEN_ON()				GPIO_SetPins(LED_PORT, GPIO_PIN_GetMask(LED_GREEN_PIN))
#	define LED_GREEN_OFF()				GPIO_ClrPins(LED_PORT, GPIO_PIN_GetMask(LED_GREEN_PIN))
#	define LED_ALL_ON()					GPIO_SetPins(LED_PORT, GPIO_PIN_GetMask(LED_GREEN_PIN) | GPIO_PIN_GetMask(LED_RED_PIN))
#	define LED_ALL_OFF()				GPIO_ClrPins(LED_PORT, GPIO_PIN_GetMask(LED_GREEN_PIN) | GPIO_PIN_GetMask(LED_RED_PIN))
#endif
#define LED_Init()						do{\
											GPIO_Dir(LED_PORT, GPIO_MODE_OUT_PP, LED_RED_PIN);\
											GPIO_Dir(LED_PORT, GPIO_MODE_OUT_PP, LED_GREEN_PIN);\
										}while(0)
#define LED_Fini()						do{\
											GPIO_Dir(LED_PORT, GPIO_MODE_IN_FLOATING, LED_RED_PIN);\
											GPIO_Dir(LED_PORT, GPIO_MODE_IN_FLOATING, LED_GREEN_PIN);\
										}while(0)

/****************************** KEY ******************************/
#if (_HARDWARE_VER == STM32_MINI)
#	define KEY_PORT						GPIOC
#	define KEY_PIN						GPIO_PIN_13
#	define KEY_IsDown()					(!GPIO_GetInPins(KEY_PORT, GPIO_PIN_GetMask(KEY_PIN)) == 0)
#	define KEY_Init()					do{ BKP_TamperPinCmd(DISABLE); GPIO_Dir(KEY_PORT, GPIO_MODE_IPU, KEY_PIN); }while(0)
#	define KEY_Fini()					do{ GPIO_Dir(KEY_PORT, GPIO_MODE_IN_FLOATING, KEY_PIN); BKP_TamperPinCmd(ENABLE); }while(0)
#elif (_HARDWARE_VER == STM32_PRO)
#	define KEY_PORT						GPIOC
#	define KEY_PIN						GPIO_PIN_13
#	define KEY_IsDown()					(GPIO_GetInPins(KEY_PORT, GPIO_PIN_GetMask(KEY_PIN)) == 0)
#	define KEY_Init()					do{ BKP_TamperPinCmd(DISABLE); GPIO_Dir(KEY_PORT, GPIO_MODE_IPU, KEY_PIN); }while(0)
#	define KEY_Fini()					do{ GPIO_Dir(KEY_PORT, GPIO_MODE_IN_FLOATING, KEY_PIN); BKP_TamperPinCmd(ENABLE); }while(0)
#elif (_HARDWARE_VER == STM32_NANO)
#	define KEY_PORT						GPIOB
#	define KEY_PIN						GPIO_PIN_9
#	define KEY_IsDown()					(GPIO_GetInPins(KEY_PORT, GPIO_PIN_GetMask(KEY_PIN)))
#	define KEY_Init()					GPIO_Dir(KEY_PORT, GPIO_MODE_IPD, KEY_PIN)
#	define KEY_Fini()					GPIO_Dir(KEY_PORT, GPIO_MODE_IN_FLOATING, KEY_PIN)
#endif


/****************************** USB *****************************/
// For USB 2.0, use DP
// For USB 1.1, use DM
#define USB_DP_PORT						GPIOA
#define USB_DP_PIN						GPIO_PIN_12

#define USB_Disable()					PowerOff()
#define USB_D_SETOUTPUT()				GPIO_Dir(USB_DP_PORT, GPIO_MODE_OUT_PP, USB_DP_PIN)
#define USB_D_SET()						GPIO_SetPins(USB_DP_PORT, GPIO_PIN_GetMask(USB_DP_PIN))
#define USB_D_CLR()						GPIO_ClrPins(USB_DP_PORT, GPIO_PIN_GetMask(USB_DP_PIN))

/*************************** Mass-prodct ***************************/
#define VERSALOON_OFFLINE_ADDR			(0x08000000 + 40 * 1024)
#define VERSALOON_OFFLINE_SIZE			(32 * 1024)
#define VERSALOON_OFFLINE_PAGE_SIZE		(1 * 1024)

/***************************** EXTERN ****************************/
#define USB_DATA_BUFF_SIZE				(8 * 1024)
#define ASYN_DATA_BUFF_SIZE				(4 * 1024)

extern SPI_InitTypeDef   SPI_InitStructure;

void SPI_Configuration(SPI_TypeDef* SPIx, u16 mode, u16 brp, u16 fb, u16 cpol, u16 cpha);

extern vu32 rep_len, cmd_len;
extern u8 buffer_out[USB_DATA_BUFF_SIZE], *buffer_in;
extern uint8 asyn_rx_buf[ASYN_DATA_BUFF_SIZE];
extern uint16 asyn_rx_idx, asyn_tx_idx;
extern vu32 count_out, usb_ovf;
extern vu32 usb_in_data_remain, usb_in_numofpackage;
extern uint16 Vtarget;

#if (USB_WITH_CDC > 0) && (USB_CDC_TARGET_IF == USB_CDC_ON_TOUCH)
extern uint8 touch_enable;
#endif

extern void System_Idle_Loop(void);
void GLOBAL_OUTPUT_Acquire(void);
void GLOBAL_OUTPUT_Release(void);

/*************************** Off-Line ***************************/
#define VERSALOON_OFFDATA_SIZE			(24 * 1024)

/**************************** Checks ****************************/
#define _HARDWARE_VER_STR				make_ver(_HARDWARE_VER)
#define make_ver(v)						make_str(v)
#define make_str(s)						# s



#if (USB_PROTOCOL != USB_ST_VCOM) && (USB_WITH_CDC != USB_WITH_IAD_CDC) && USB_WITH_MASSSTORAGE
#	warning "MSD will not be enabled, if not using IAD for CDC"
#endif

#if USB_PROTOCOL == USB_ST_VCOM
#	define USB_RX_DOUBLEBUFFER_EN		1
#	define USB_TX_DOUBLEBUFFER_EN		0//1
#else
#	define USB_RX_DOUBLEBUFFER_EN		0
#	define USB_TX_DOUBLEBUFFER_EN		0
#endif

#if USB_TO_CAN_EN == 1
#	warning "USB_TO_CAN_EN can not supported by this hardware"
#	undef USB_TO_CAN_EN
#	define USB_TO_CAN_EN				0
#endif

#if USB_TO_XXX_EN == 0
#	define USB_TO_USART_EN				0
#	define USB_TO_SPI_EN				0
#	define USB_TO_I2C_EN				0
#	define USB_TO_GPIO_EN				0
#	define USB_TO_CAN_EN				0
#	define USB_TO_PWM_EN				0
#	define USB_TO_ADC_EN				0
#	define USB_TO_DAC_EN				0
#	define USB_TO_POWER_EN				0
#endif

#if MP_EN == 0
#	define MP_PSOC_EN					0
#	define MP_C8051F_C2_EN				0
#	define MP_C8051F_JTAG_EN			0
#	define MP_S5X_EN					0
#	define MP_STM8_EN					0
#endif

#define INTERFACE_C2_EN					((USB_TO_XXX_EN && USB_TO_C2_EN) || (MP_EN && MP_C8051F_C2_EN))
#define INTERFACE_I2C_EN				(USB_TO_XXX_EN && USB_TO_I2C_EN)
#define INTERFACE_ISSP_EN				((USB_TO_XXX_EN && USB_TO_ISSP_EN) || (MP_EN && MP_PSOC_EN))
#define INTERFACE_LPC_ICP_EN			((USB_TO_XXX_EN && USB_TO_LPCICP_EN) || (MP_EN && MP_LPCICP_EN))
#define INTERFACE_JTAG_EN				((USB_TO_XXX_EN && (USB_TO_JTAG_HL_EN || USB_TO_JTAG_LL_EN)) || AVR_JTAG_EN || AVR32_JTAG_EN || AVRX_JTAG_EN || VSLLINK_EN || (MP_EN && MP_C8051F_JTAG_EN))
#define INTERFACE_MSP430_JTAG_EN		USB_TO_MSP430_JTAG_EN
#define INTERFACE_SBW_EN				(USB_TO_XXX_EN && USB_TO_SBW_EN)
#define INTERFACE_SPI_EN				((USB_TO_XXX_EN && USB_TO_SPI_EN) || AVR_ISP_EN || (MP_EN && MP_S5X_EN))
#define INTERFACE_SWIM_EN				((USB_TO_XXX_EN && USB_TO_SWIM_EN) || (MP_EN && MP_STM8_EN))
#define INTERFACE_USART_EN				(USB_TO_XXX_EN && USB_TO_USART_EN)
